@@ -57,6 +57,9 @@ class XEP_0030(BasePlugin):
     Given   Given   A single node
     ====== ======= ============================
 
+    Adding information for a given node without specifying the JID will
+    use the bound JID and therefore must be done after the bind happens.
+
     Stream Handlers:
 
     ::
@@ -307,7 +310,7 @@ class XEP_0030(BasePlugin):
         return self.api['has_identity'](jid, node, ifrom, data)
 
     async def get_info_from_domain(self, domain=None, timeout=None,
-                                   cached=True, callback=None):
+                                   cached=True, callback=None, **iqkwargs):
         """Fetch disco#info of specified domain and one disco#items level below
         """
 
@@ -315,13 +318,13 @@ class XEP_0030(BasePlugin):
             domain = self.xmpp.boundjid.domain
 
         if not cached or domain not in self.domain_infos:
-            infos = [self.get_info(
-                domain, timeout=timeout)]
+            infos = [asyncio.create_task(self.get_info(
+                domain, timeout=timeout, **iqkwargs))]
             iq_items = await self.get_items(
-                domain, timeout=timeout)
+                domain, timeout=timeout, **iqkwargs)
             items = iq_items['disco_items']['items']
             infos += [
-                self.get_info(item[0], timeout=timeout)
+                asyncio.create_task(self.get_info(item[0], timeout=timeout, **iqkwargs))
                 for item in items]
             info_futures, _ = await asyncio.wait(
                 infos,
@@ -385,6 +388,8 @@ class XEP_0030(BasePlugin):
                 local = True
 
         ifrom = kwargs.pop('ifrom', None)
+        if self.xmpp.is_component and ifrom is None:
+            ifrom = self.xmpp.boundjid
         if local:
             log.debug("Looking up local disco#info data "
                       "for %s, node %s.", jid, node)
@@ -455,9 +460,12 @@ class XEP_0030(BasePlugin):
                          the XEP-0059 plugin, if the plugin is loaded.
                          Otherwise the parameter is ignored.
         """
+        if ifrom is None and self.xmpp.is_component:
+            ifrom = self.xmpp.boundjid.bare
+
         if local or local is None and jid is None:
             items = await self.api['get_items'](jid, node, ifrom, kwargs)
-            return self._wrap(kwargs.get('ifrom', None), jid, items)
+            return self._wrap(ifrom, jid, items)
 
         iq = self.xmpp.Iq()
         # Check dfrom parameter for backwards compatibility
